require 'pin/error'
require 'spec_helper'

describe 'error' do
  it 'requires a status and json body' do
    expect { Pin::Error.new() }.to raise_error(ArgumentError)
  end

  context 'valid values' do
    let(:status) { '402' }
    let(:json_body) { '{"error":"card_declined","error_description":"The card was declined","charge_token":"ch_X3TI-S1LGFmo30zNc4GUaw"}' }

    subject { Pin::Error.new(status, json_body) }

    it 'should have the correct status' do
      expect(subject.status).to eq('402')
    end

    it 'should have the correct error description' do
      expect(subject.error_description).to eq('The card was declined')
    end

    it 'should have the correct error' do
      expect(subject.error).to eq('card_declined')
    end

    it 'should have the correct details' do
      expect(subject.details).to eq({'error' => 'card_declined', 'error_description' => 'The card was declined', 'charge_token' => 'ch_X3TI-S1LGFmo30zNc4GUaw'})
    end

    it 'should have the correct string representation' do
      expect(subject.to_s).to eq('HTTP Error 402: card_declined')
    end
  end
end